#!/bin/bash

# Validate JSON file argument
if [ "$#" -ne 1 ]; then
  echo "Usage: $0 <config.json>"
  exit 1
fi

JSON_FILE="$1"

# Validate JSON file existence
if [ ! -f "$JSON_FILE" ]; then
  echo "JSON file not found: $JSON_FILE"
  exit 1
fi

# Parse JSON file
BASE_URL=$(jq -r '.base_url' "$JSON_FILE")
API_KEY=$(jq -r '.api_key' "$JSON_FILE")
PROJECT_NAME=$(jq -r '.project_name' "$JSON_FILE")
CLUSTER_NAME=$(jq -r '.cluster_name' "$JSON_FILE")
COMMAND=$(jq -r '.command' "$JSON_FILE")
TIMEOUT=$(jq -r '.timeout' "$JSON_FILE")

# Validate parsed JSON values
if [ -z "$BASE_URL" ] || [ -z "$API_KEY" ] || [ -z "$PROJECT_NAME" ] || [ -z "$CLUSTER_NAME" ] || [ -z "$COMMAND" ]; then
  echo "Invalid or missing values in JSON file"
  exit 1
fi

PROJECT_ID=$(curl -s -X GET "https://${BASE_URL}/auth/v1/projects/?limit=1000&offset=0&order=ASC&orderby=name&q=" \
  -H "X-RAFAY-API-KEYID: ${API_KEY}" \
  | jq -r --arg name "${PROJECT_NAME}" '.results[] | select(.name == $name) | .id')

if [ -z "$PROJECT_ID" ]; then
  echo "Project ID not found for project name $PROJECT_NAME"
  exit 1
fi
echo "Project ID fetched successfully: $PROJECT_ID"

CLUSTER_ID=$(curl -s -X GET "https://${BASE_URL}/edge/v1/projects/$PROJECT_ID/edges/?limit=25&offset=0&q=" \
  -H "X-RAFAY-API-KEYID: ${API_KEY}" \
  | jq -r --arg name "${CLUSTER_NAME}" '.results[] | select(.name == $name) | .id')

if [ -z "$CLUSTER_ID" ]; then
  echo "Cluster ID not found for cluster name $CLUSTER_NAME"
  exit 1
fi
echo "Cluster ID fetched successfully: $CLUSTER_ID"

POST_RESPONSE=$(curl -s -X POST \
  "https://${BASE_URL}/cmdexec/v1/projects/$PROJECT_ID/edges/$CLUSTER_ID/execute/" \
  -H "X-RAFAY-API-KEYID: ${API_KEY}" \
  -d "{\"target_type\": \"cluster\", \"command\": \"$COMMAND\", \"timeout\": $TIMEOUT}")

EXEC_ID=$(echo "$POST_RESPONSE" | jq -r '.Id')

if [ -z "$EXEC_ID" ] || [ "$EXEC_ID" == "null" ]; then
  echo "Failed to retrieve execution ID. Response: $POST_RESPONSE"
  exit 1
fi
echo "Execution ID retrieved successfully: $EXEC_ID"

GET_RESPONSE=$(curl -s -X GET \
  "https://${BASE_URL}/cmdexec/v1/projects/$PROJECT_ID/edges/$CLUSTER_ID/execution/$EXEC_ID/" \
  -H "X-RAFAY-API-KEYID: ${API_KEY}")

RETURN_FIELD=$(echo "$GET_RESPONSE" | jq -r '.NodeResponses[0].Resp.Return')

if [ -z "$RETURN_FIELD" ] || [ "$RETURN_FIELD" == "null" ]; then
  echo "Failed to retrieve the Return field. Response: $GET_RESPONSE"
  exit 1
fi

echo "{\"command_output\": $RETURN_FIELD}"
